# coding=utf-8
from __future__ import absolute_import, print_function

from suanpan.g import g
from suanpan.run import cli, env, run

__version__ = "0.12.14"


if __name__ == "__main__":
    print(f"Suanpan SDK (ver: {__version__})")
